#
# Copyright 2019 BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
import os

from cliboa.core.validator import EssentialParameters
from cliboa.scenario.base import BaseStep
from cliboa.util.constant import StepStatus
from cliboa.util.sftp import Sftp


class SftpBaseLoad(BaseStep):
    def __init__(self):
        super().__init__()
        self._src_dir = ""
        self._src_pattern = ""
        self._dest_dir = ""
        self._host = None
        self._port = 22
        self._user = None
        self._password = None
        self._key = None
        self._timeout = 30
        self._retry_count = 3

    def src_dir(self, src_dir):
        self._src_dir = src_dir

    def src_pattern(self, src_pattern):
        self._src_pattern = src_pattern

    def dest_dir(self, dest_dir):
        self._dest_dir = dest_dir

    def host(self, host):
        self._host = host

    def port(self, port):
        self._port = port

    def user(self, user):
        self._user = user

    def password(self, password):
        self._password = password

    def key(self, key):
        self._key = key

    def timeout(self, timeout):
        self._timeout = timeout

    def retry_count(self, retry_count):
        self._retry_count = retry_count


# deprecated
class SftpFileLoad(SftpBaseLoad):
    """
    Upload file to sftp server
    """

    def __init__(self):
        SftpBaseLoad.__init__(self)

    def execute(self, *args):
        # essential parameters check
        valid = EssentialParameters(
            self.__class__.__name__,
            [self._host, self._user, self._src_dir, self._src_pattern, self._dest_dir],
        )
        valid()

        sftp = Sftp(
            self._host,
            self._user,
            self._password,
            self._key,
            self._timeout,
            self._retry_count,
            self._port,
        )

        files = super().get_target_files(self._src_dir, self._src_pattern)
        for file in files:
            sftp.put_file(file, os.path.join(self._dest_dir, os.path.basename(file)))


class SftpUpload(SftpBaseLoad):
    """
    Upload file to sftp server
    """

    def __init__(self):
        super().__init__()
        self._quit = False

    def quit(self, quit):
        self._quit = quit

    def execute(self, *args):
        # essential parameters check
        valid = EssentialParameters(
            self.__class__.__name__,
            [self._host, self._user, self._src_dir, self._src_pattern, self._dest_dir],
        )
        valid()

        sftp = Sftp(
            self._host,
            self._user,
            self._password,
            self._key,
            self._timeout,
            self._retry_count,
            self._port,
        )
        files = super().get_target_files(self._src_dir, self._src_pattern)

        if len(files) > 0:
            for file in files:
                sftp.put_file(
                    file, os.path.join(self._dest_dir, os.path.basename(file))
                )
                self._logger.info("%s is successfully uploaded." % file)
        else:
            self._logger.info(
                "Files to upload do not exist. File pattern: {}".format(
                    os.path.join(self._src_dir, self._src_pattern)
                )
            )
            if self._quit is True:
                return StepStatus.SUCCESSFUL_TERMINATION



#
# Copyright 2019 BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
import os

from cliboa.scenario.aws import BaseS3
from cliboa.scenario.validator import EssentialParameters
from cliboa.util.constant import StepStatus


class S3Upload(BaseS3):
    """
    Upload to S3
    """

    def __init__(self):
        super().__init__()
        self._key = None
        self._src_dir = None
        self._src_pattern = None
        self._quit = False

    def key(self, key):
        self._key = key

    def src_dir(self, src_dir):
        self._src_dir = src_dir

    def src_pattern(self, src_pattern):
        self._src_pattern = src_pattern

    def quit(self, quit):
        self._quit = quit

    def execute(self, *args):
        super().execute()

        valid = EssentialParameters(
            self.__class__.__name__, [self._src_dir, self._src_pattern]
        )
        valid()

        resource = self._s3_resource()
        bucket = resource.Bucket(self._bucket)
        files = super().get_target_files(self._src_dir, self._src_pattern)

        if len(files) > 0:
            for f in files:
                bucket.upload_file(
                    Key=os.path.join(self._key, os.path.basename(f)), Filename=f
                )
        else:
            self._logger.info(
                "Files to upload do not exist. File pattern: {}".format(
                    os.path.join(self._src_dir, self._src_pattern)
                )
            )
            if self._quit is True:
                return StepStatus.SUCCESSFUL_TERMINATION



#
# Copyright BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
import codecs
import csv
import os


class Csv(object):
    @staticmethod
    def extract_columns_with_names(
        input_file, output_file, remain_column_names, enc="utf-8",
    ):
        """
        Extract only the necessary columns from a CSV file and output a new CSV

        Args:
            input_file: Input csv file name
            output_file: Output csv file name
            remain_column_names: Columns which remain
            enc: Encording
        """
        with codecs.open(input_file, mode="r", encoding=enc) as in_f, codecs.open(
            output_file + ".tmp", mode="w", encoding=enc
        ) as out_f:
            reader = csv.DictReader(in_f)
            writer = csv.writer(out_f)
            writer.writerow(remain_column_names)
            for row in reader:
                contents = []
                for c in remain_column_names:
                    contents.append(row[c])
                writer.writerow(contents)
            out_f.flush()
        os.remove(input_file)
        os.rename(output_file + ".tmp", input_file)

    @staticmethod
    def extract_columns_with_numbers(
        input_file, output_file, remain_column_numbers, enc="utf-8"
    ):
        """
        Extract only the necessary columns from a CSV file and output a new CSV

        Args:
            input_file: Input csv file name
            output_file: Output csv file name
            remain_column_numbers: Column numbers which remain
            enc: Encording
        """
        with codecs.open(input_file, mode="r", encoding=enc) as in_f, codecs.open(
            output_file + ".tmp", mode="w", encoding=enc
        ) as out_f:
            reader = csv.reader(in_f)
            writer = csv.writer(out_f)
            for i, row in enumerate(reader):
                contents = []
                for j, r in enumerate(row):
                    if (j + 1) not in remain_column_numbers:
                        continue
                    contents.append(r)
                writer.writerow(contents)
            out_f.flush()
        os.remove(input_file)
        os.rename(output_file + ".tmp", input_file)



#
# Copyright BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#

from cliboa.scenario.base import BaseStep
from cliboa.scenario.validator import EssentialParameters


class BaseAzure(BaseStep):
    """
    Base class of Azure related classes
    """

    def __init__(self):
        super().__init__()
        self._account_url = None
        self._account_access_key = None
        self._connection_string = None

    def account_url(self, account_url):
        self._account_url = account_url

    def account_access_key(self, account_access_key):
        self._account_access_key = account_access_key

    def connection_string(self, connection_string):
        self._connection_string = connection_string


class BaseAzureBlob(BaseAzure):
    """
    Base class of Azure Blob Storage related classes
    """

    def __init__(self):
        super().__init__()
        self._container_name = None

    def container_name(self, container_name):
        self._container_name = container_name

    def execute(self, *args):
        super().execute()
        valid = EssentialParameters(self.__class__.__name__, [self._container_name])
        valid()



#
# Copyright 2019 BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
import csv
from glob import glob

from cliboa.scenario.base import BaseStep
from cliboa.scenario.validator import EssentialParameters, IOInput
from cliboa.util.cache import ObjectStore
from cliboa.util.exception import CliboaException, FileNotFound


class FileRead(BaseStep):
    """
    The parent class to read the specified file
    """

    def __init__(self):
        super().__init__()
        self._src_path = None
        self._src_dir = None
        self._src_pattern = None
        self._encoding = "utf-8"

    def src_path(self, src_path):
        self._src_path = src_path

    def src_dir(self, src_dir):
        self._src_dir = src_dir

    def src_pattern(self, src_pattern):
        self._src_pattern = src_pattern

    def encoding(self, encoding):
        self._encoding = encoding

    def execute(self, *args):
        # essential parameters check
        valid = EssentialParameters(
            self.__class__.__name__, [self._src_dir, self._src_pattern]
        )
        valid()


class CsvRead(FileRead):
    """
    Read the specified csv file
    """

    LOAD_CHUNK_SIZE = 10000

    def __init__(self):
        super().__init__()
        self._columns = None

    def columns(self, columns):
        self._columns = columns

    def execute(self, *args):
        input_valid = IOInput(self._io)
        input_valid()

        files = glob(self._src_path)
        if len(files) > 1:
            raise CliboaException("Input file must be only one.")

        if len(files) == 0:
            raise FileNotFound("The specified csv file not found.")

        with open(files[0], "r", encoding=self._encoding) as f:

            # save per one column
            if self._columns:
                reader = csv.DictReader(f, delimiter=",")
                for row in reader:
                    # extract only the specified columns
                    row_dict = {}
                    for c in self._columns:
                        if not row.get(c):
                            continue
                        row_dict[c] = row.get(c)
                    self._s.save(row_dict)
            else:
                reader = csv.reader(f)
                header = next(reader, None)
                for row in reader:
                    row_dict = dict(zip(header, row))
                    self._s.save(row_dict)

        # cache downloaded file names
        ObjectStore.put(self._step, files)



#!/usr/bin/env python
#
# Copyright 2019 BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# Always prefer setuptools over distutils
from io import open
from os import path

from setuptools import setup


def read(filename):
    with open(path.join(path.dirname(__file__), filename), encoding="utf-8") as f:
        return f.read()


setup(
    name="cliboa",
    version="1.3.3beta",
    description="application framework for ETL(ELT) processing",
    # long_description=read("README.md"),
    # long_description_content_type='text/markdown',
    url="https://github.com/BrainPad/cliboa",  # Optional
    author="BrainPad",
    # author_email='brainpad.co.jp',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    packages=[
        "cliboa",
        "cliboa.adapter",
        "cliboa.cli",
        "cliboa.common",
        "cliboa.conf",
        "cliboa.core",
        "cliboa.scenario",
        "cliboa.util",
        "cliboa.scenario.extract",
        "cliboa.scenario.transform",
        "cliboa.scenario.load",
        "cliboa.template",
        "cliboa.template.bin",
    ],
    package_data={
        "cliboa.conf": ["logging.conf", "cliboa.ini"],
        "cliboa.template": [
            "Pipfile.above35",
            "Pipfile.above36",
            "Pipfile.above37",
        ],
    },
    python_requires=">=3.5",
    entry_points={"console_scripts": ["cliboadmin = cliboa.cli.cliboadmin:main"]},
)
